@@ -41,6 +41,7 @@ macro_rules! test_hammerfest_store_pg {
     register_test!($(#[$meta])*, $api, test_touch_godchildren);
     register_test!($(#[$meta])*, $api, test_touch_hammerfest_shop);
     register_test!($(#[$meta])*, $api, test_touch_hammerfest_inventory);
+    register_test!($(#[$meta])*, $api, test_touch_hammerfest_inventory_twice);
   };
 }
 
@@ -199,8 +200,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: None,
-        profile: HammerfestProfile {
+        session: None,
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -215,7 +216,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -232,8 +233,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: None,
-        profile: HammerfestProfile {
+        session: None,
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -248,7 +249,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -258,8 +259,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: None,
-        profile: HammerfestProfile {
+        session: None,
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -274,7 +275,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -290,8 +291,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: None,
-        profile: HammerfestProfile {
+        session: None,
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -306,7 +307,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -316,8 +317,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: None,
-        profile: HammerfestProfile {
+        session: None,
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -332,7 +333,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -348,8 +349,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: None,
-        profile: HammerfestProfile {
+        session: None,
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -364,7 +365,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -374,8 +375,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: None,
-        profile: HammerfestProfile {
+        session: None,
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -390,7 +391,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -400,8 +401,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: None,
-        profile: HammerfestProfile {
+        session: None,
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -416,7 +417,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -441,8 +442,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: Some(alice.clone()),
-        profile: HammerfestProfile {
+        session: Some(alice.clone()),
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -457,7 +458,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -482,8 +483,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: Some(alice.clone()),
-        profile: HammerfestProfile {
+        session: Some(alice.clone()),
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -498,7 +499,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -508,8 +509,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: Some(alice.clone()),
-        profile: HammerfestProfile {
+        session: Some(alice.clone()),
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -524,7 +525,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -549,8 +550,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: Some(alice.clone()),
-        profile: HammerfestProfile {
+        session: Some(alice.clone()),
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -565,7 +566,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -575,8 +576,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: Some(alice.clone()),
-        profile: HammerfestProfile {
+        session: Some(alice.clone()),
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -591,7 +592,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -601,8 +602,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: Some(alice.clone()),
-        profile: HammerfestProfile {
+        session: Some(alice.clone()),
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -617,7 +618,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -642,8 +643,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: Some(alice.clone()),
-        profile: HammerfestProfile {
+        session: Some(alice.clone()),
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -658,7 +659,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -668,8 +669,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: Some(alice.clone()),
-        profile: HammerfestProfile {
+        session: Some(alice.clone()),
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "234".parse().unwrap(),
@@ -684,7 +685,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -694,8 +695,8 @@ where
     let actual = api
       .hammerfest_store
       .touch_profile(&HammerfestProfileResponse {
-        session_user: Some(alice.clone()),
-        profile: HammerfestProfile {
+        session: Some(alice.clone()),
+        profile: Some(HammerfestProfile {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
             id: "123".parse().unwrap(),
@@ -710,7 +711,7 @@ where
           hall_of_fame: None,
           items: Default::default(),
           quests: Default::default(),
-        },
+        }),
       })
       .await;
     assert_ok!(actual);
@@ -761,7 +762,7 @@ where
     let actual = api
       .hammerfest_store
       .touch_theme_page(&HammerfestForumThemePageResponse {
-        session_user: Some(alice.clone()),
+        session: Some(alice.clone()),
         page: HammerfestForumThemePage {
           theme: ShortHammerfestForumTheme {
             server: HammerfestServer::HammerfestFr,
@@ -807,7 +808,7 @@ where
     let actual = api
       .hammerfest_store
       .touch_thread_page(&HammerfestForumThreadPageResponse {
-        session_user: None,
+        session: None,
         page: HammerfestForumThreadPage {
           theme: ShortHammerfestForumTheme {
             server: HammerfestServer::HammerfestFr,
@@ -895,7 +896,7 @@ pub(crate) async fn test_touch_forum_thread_page_as_moderator<TyClock, TyHammerf
     let actual = api
       .hammerfest_store
       .touch_thread_page(&HammerfestForumThreadPageResponse {
-        session_user: Some(alice),
+        session: Some(alice),
         page: HammerfestForumThreadPage {
           theme: ShortHammerfestForumTheme {
             server: HammerfestServer::HammerfestFr,
@@ -982,7 +983,7 @@ where
     let actual = api
       .hammerfest_store
       .touch_godchildren(&HammerfestGodchildrenResponse {
-        session_user: alice.clone(),
+        session: alice.clone(),
         godchildren: vec![],
       })
       .await;
@@ -994,7 +995,7 @@ where
     let actual = api
       .hammerfest_store
       .touch_godchildren(&HammerfestGodchildrenResponse {
-        session_user: alice.clone(),
+        session: alice.clone(),
         godchildren: vec![HammerfestGodchild {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
@@ -1013,7 +1014,7 @@ where
     let actual = api
       .hammerfest_store
       .touch_godchildren(&HammerfestGodchildrenResponse {
-        session_user: alice.clone(),
+        session: alice.clone(),
         godchildren: vec![
           HammerfestGodchild {
             user: ShortHammerfestUser {
@@ -1042,7 +1043,7 @@ where
     let actual = api
       .hammerfest_store
       .touch_godchildren(&HammerfestGodchildrenResponse {
-        session_user: alice.clone(),
+        session: alice.clone(),
         godchildren: vec![HammerfestGodchild {
           user: ShortHammerfestUser {
             server: HammerfestServer::HammerfestFr,
@@ -1061,7 +1062,7 @@ where
     let actual = api
       .hammerfest_store
       .touch_godchildren(&HammerfestGodchildrenResponse {
-        session_user: alice.clone(),
+        session: alice.clone(),
         godchildren: vec![
           HammerfestGodchild {
             user: ShortHammerfestUser {
@@ -1090,7 +1091,7 @@ where
     let actual = api
       .hammerfest_store
       .touch_godchildren(&HammerfestGodchildrenResponse {
-        session_user: alice.clone(),
+        session: alice.clone(),
         godchildren: vec![
           HammerfestGodchild {
             user: ShortHammerfestUser {
@@ -1133,7 +1134,7 @@ where
     let actual = api
       .hammerfest_store
       .touch_shop(&HammerfestShopResponse {
-        session_user: alice.clone(),
+        session: alice.clone(),
         shop: HammerfestShop {
           weekly_tokens: 0,
           purchased_tokens: None,
@@ -1149,7 +1150,7 @@ where
     let actual = api
       .hammerfest_store
       .touch_shop(&HammerfestShopResponse {
-        session_user: alice.clone(),
+        session: alice.clone(),
         shop: HammerfestShop {
           weekly_tokens: 0,
           purchased_tokens: Some(5),
@@ -1165,7 +1166,7 @@ where
     let actual = api
       .hammerfest_store
       .touch_shop(&HammerfestShopResponse {
-        session_user: alice.clone(),
+        session: alice.clone(),
         shop: HammerfestShop {
           weekly_tokens: 0,
           purchased_tokens: Some(5),
@@ -1181,7 +1182,7 @@ where
     let actual = api
       .hammerfest_store
       .touch_shop(&HammerfestShopResponse {
-        session_user: alice.clone(),
+        session: alice.clone(),
         shop: HammerfestShop {
           weekly_tokens: 0,
           purchased_tokens: Some(5),
@@ -1212,7 +1213,53 @@ pub(crate) async fn test_touch_hammerfest_inventory<TyClock, TyHammerfestStore>(
     let actual = api
       .hammerfest_store
       .touch_inventory(&HammerfestInventoryResponse {
-        session_user: alice.clone(),
+        session: alice.clone(),
+        inventory: {
+          let mut inventory = HashMap::new();
+          inventory.insert("1000".parse().unwrap(), 10);
+          inventory
+        },
+      })
+      .await;
+    assert_ok!(actual);
+  }
+}
+
+pub(crate) async fn test_touch_hammerfest_inventory_twice<TyClock, TyHammerfestStore>(
+  api: TestApi<TyClock, TyHammerfestStore>,
+) where
+  TyClock: ApiRef<VirtualClock>,
+  TyHammerfestStore: HammerfestStore,
+{
+  let alice = HammerfestSessionUser {
+    user: ShortHammerfestUser {
+      server: HammerfestServer::HammerfestFr,
+      id: "123".parse().unwrap(),
+      username: "alice".parse().unwrap(),
+    },
+    tokens: 50,
+  };
+  api.clock.as_ref().advance_to(Utc.ymd(2021, 1, 1).and_hms(0, 0, 0));
+  {
+    let actual = api
+      .hammerfest_store
+      .touch_inventory(&HammerfestInventoryResponse {
+        session: alice.clone(),
+        inventory: {
+          let mut inventory = HashMap::new();
+          inventory.insert("1000".parse().unwrap(), 10);
+          inventory
+        },
+      })
+      .await;
+    assert_ok!(actual);
+  }
+  api.clock.as_ref().advance_by(Duration::seconds(1));
+  {
+    let actual = api
+      .hammerfest_store
+      .touch_inventory(&HammerfestInventoryResponse {
+        session: alice.clone(),
         inventory: {
           let mut inventory = HashMap::new();
           inventory.insert("1000".parse().unwrap(), 10);
